/*  GnomeKiss - A KiSS viewer for the GNOME desktop
    Copyright (C) 2000  Nick Lamb <njl195@zepler.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "kiss.h"

static KissCell *target;
static int x1, y1;

void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	lha_close();
	gtk_main_quit();
}


gboolean
on_application_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  lha_close();
  gtk_main_quit();
  return FALSE;
}


gboolean
on_area_expose_event                   (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  gint32 y, height;
  guchar *buffer;

  if (rgb_buf == NULL) return FALSE;

  if (global_lock == KISS_PENDING) {
    /* Clear background */
    buffer= rgb_buf;
    while (buffer < rgb_buf + rgb_buf_stride * rgb_buf_height) {
      *(buffer++)= palettes[0][0];
      *(buffer++)= palettes[0][1];
      *(buffer++)= palettes[0][2];
    }
    g_slist_foreach (cells, (GFunc) cell_render, NULL);
  }

  y= event->area.y;
  height= (y + event->area.height <= rgb_buf_height) ? event->area.height
                                     : (rgb_buf_height - y);

  gdk_gc_set_clip_rectangle (widget->style->fg_gc[widget->state], NULL);
  gdk_draw_rgb_image (widget->window, widget->style->fg_gc[widget->state],
                      0, y, rgb_buf_width, height, GDK_RGB_DITHER_NORMAL,
                      rgb_buf + (y * rgb_buf_stride), rgb_buf_stride);

  global_lock= KISS_DONE; /* Rendering done */
  return TRUE;
}


gboolean
on_area_configure_event                (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{
  return FALSE;
}


void
on_properties_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_set_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gchar *name;
  name = gtk_widget_get_name((GtkWidget *)menuitem);
  switch_view (name[4] - '0');
  render_all();
}


void
on_preferences_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (about);
}

gboolean
on_area_button_press_event             (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  x1= (int) event->x; y1= (int) event->y;
  target= intersect(x1, y1);

  if (target == NULL) {
    change_cursor(widget->window, GDK_HAND1);
    return FALSE;
  }

  change_cursor(widget->window, GDK_HAND2);

  x1-= target->object->x[view];
  y1-= target->object->y[view];

  events(target->press);
  events(target->object->press);
  if (target->object->sticky) {
    events(target->object->fixcatch);
  } else {
    events(target->object->catch);
  }

  return FALSE;
}

gboolean
on_area_motion_notify_event            (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
  int x2, y2;

  if (target == NULL)
    return FALSE;

  gdk_window_get_pointer(event->window, &x2, &y2, NULL);
  if (!target->object->sticky) {
    object_move(target->object, x2 - x1, y2 - y1);
    render_all();
  }

  return FALSE;
}

gboolean
on_area_button_release_event           (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  int x2, y2;
  x2= (int) event->x; y2= (int) event->y;

  if (target == NULL)
    return FALSE;

  if (target->object->sticky) {
    events(target->object->fixdrop);
  } else {
    events(target->object->drop);
  }

  if (target->object->sticky) {
    target->object->sticky--;
    if (!target->object->sticky) events(target->object->unfix);
  } else {
    object_move(target->object, x2 - x1, y2 - y1);
    render_all();
  }

  events(target->release);
  events(target->object->release);

  change_cursor(widget->window, GDK_LEFT_PTR);

  return FALSE;
}

void
on_open_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_file_selection_complete(GTK_FILE_SELECTION(set_open), "*.lzh");
  gtk_widget_show (set_open);
}


void
on_set_open_ok                         (GtkButton       *button,
                                        gpointer         user_data)
{
  char *filename;

  gtk_widget_hide (set_open);
  filename= gtk_file_selection_get_filename(GTK_FILE_SELECTION(set_open));
  open_either(filename);
}


void
on_set_open_cancel                     (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (set_open);
}

void
on_open_config_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_file_selection_complete(GTK_FILE_SELECTION(config_open), "*.cnf");
  gtk_widget_show (config_open);
}


void
on_config_open_ok                      (GtkButton       *button,
                                        gpointer         user_data)
{
  gchar *filename;
  gtk_widget_hide (config_open);
  filename= gtk_file_selection_get_filename(GTK_FILE_SELECTION(config_open));
  parse_file(filename);
}


void
on_config_open_cancel                  (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (config_open);
}

void
on_set_clicked                         (GtkButton       *button,
                                        gpointer         user_data)
{
  gchar *name;
  name = gtk_widget_get_name((GtkWidget *)button);
  switch_view (name[8] - '0');
  render_all();
}

