/*  GnomeKiss - A KiSS viewer for the GNOME desktop
    Copyright (C) 2000  Nick Lamb <njl195@zepler.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "kiss.h"

KissCollision *collision_find(KissCell *cell1, KissCell *cell2) {
  KissCollision *collision;
  GSList *list= cell1->object->collisions;

  for (; list != NULL; list= g_slist_next(list)) {
    collision= (KissCollision *) list->data;
    if (collision->cell1 == cell1 && collision->cell2 == cell2)
      return collision;
  }

  /* Create a new one instead */
  
  collision= g_new0(KissCollision, 1);
  collision->cell1= cell1;
  collision->cell2= cell2;
  cell1->object->collisions=
    g_slist_prepend (cell1->object->collisions, collision);
  cell2->object->collisions=
    g_slist_prepend (cell2->object->collisions, collision);

  return collision;
}

int cell_collision(KissCell *cell1, KissCell *cell2) {

  if (!cell1->mapped || !(cell1->visible & (1 << view)))
    return 0;

  if (!cell2->mapped || !(cell2->visible & (1 << view)))
    return 0;

  if (cell1->x[view] >= cell2->x[view] + cell2->width)
    return 0;
  if (cell1->x[view] + cell1->width <= cell2->x[view])
    return 0;
  if (cell1->y[view] >= cell2->y[view] + cell2->height)
    return 0;
  if (cell1->y[view] + cell1->height <= cell2->y[view])
    return 0;
  else
    return 1;
}

