#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>

#define ALARMS 1024
#define OBJECTS 2000
#define SETS 10
#define PALETTES 256

#define KISS_PENDING 0
#define KISS_DONE 1

extern GtkWidget *app, *area, *appbar, *about, *set_open, *config_open;
extern GtkWidget *buttons[SETS], *items[SETS];
extern gint32 rgb_buf_width, rgb_buf_height, rgb_buf_stride;

extern guchar *rgb_buf;
extern int view, hasbgcolor;
extern int global_lock; /* if KISS_PENDING, redraw will refresh rgb_buf */

typedef GSList *KissActionList;

typedef struct {
  int x[SETS], y[SETS];      /* co-ordinatess */
  int width, height;         /* Object size */
  KissActionList cells;      /* KissCells in this object */
  int sticky;                /* +ve values indicate stuck */

  KissActionList catch;      /* fKiSS catch(#n) */
  KissActionList drop;       /* fKiSS drop(#n) */
  KissActionList fixcatch;   /* fKiSS fixcatch(#n) */
  KissActionList fixdrop;    /* fKiSS fixdrop(#n) */
  KissActionList press;      /* fKiSS press(#n) */
  KissActionList release;    /* fKiSS release(#n) */
  KissActionList unfix;      /* fKiSS unfix(#n) */

  GSList *collisions;        /* fKiSS collide("NAME.CEL") apart("NAME.CEL") */
} KissObject;
 
typedef struct {
  guchar *rgb, *mask;        /* Image buffers */
  guchar *name;              /* Name of CEL file */
  KissObject *object;

  int x[SETS], y[SETS];      /* co-ords */
  int width, height;         /* Image size */

  int offx, offy;            /* offset co-ordinatess */

  unsigned int visible;      /* 1 << N FALSE if hidden */
  unsigned int mapped;       /* boolean map/ unmap */
  int alpha;                 /* cell-level alpha */

  KissActionList catch;      /* fKiSS catch("NAME.CEL") */
  KissActionList drop;       /* fKiSS drop("NAME.CEL") */
  KissActionList fixcatch;   /* fKiSS fixcatch("NAME.CEL") */
  KissActionList fixdrop;    /* fKiSS fixdrop("NAME.CEL") */
  KissActionList press;      /* fKiSS press("NAME.CEL") */
  KissActionList release;    /* fKiSS release("NAME.CEL") */
} KissCell;

typedef enum {
  ACTION_ALTMAP_CELL,
  ACTION_ALTMAP_OBJECT,
  ACTION_CHANGESET,
  ACTION_CHANGECOL,
  ACTION_DEBUG,
  ACTION_MAP_CELL,
  ACTION_MAP_OBJECT,
  ACTION_MOVE_OBJECT,
  ACTION_NOP,
  ACTION_QUIT,
  ACTION_RANDOMTIMER,
  ACTION_SHELL,
  ACTION_SOUND,
  ACTION_TIMER,
  ACTION_TRANSPARENT_CELL,
  ACTION_TRANSPARENT_OBJECT,
  ACTION_UNMAP_CELL,
  ACTION_UNMAP_OBJECT,
  ACTION_VIEWPORT,
  ACTION_WINDOW_SIZE,
/* FKiSS 2 */
  ACTION_MOVE_BYX,
  ACTION_MOVE_BYY,
  ACTION_MOVE_TO,
  ACTION_MUSIC,
  ACTION_NOTIFY,
/* FKiSS 2.1 */
  ACTION_IF_FIXED,
  ACTION_IF_MAPPED,
  ACTION_IF_MOVED,
  ACTION_IF_NOT_FIXED,
  ACTION_IF_NOT_MAPPED,
  ACTION_IF_NOT_MOVED,
  ACTION_MOVE_RANDX,
  ACTION_MOVE_RANDY,
  ACTION_MOVE_TORAND,
  ACTION_SETFIX,
/* FKiSS 3.0 */
  ACTION_VAR_LET,
  ACTION_VAR_ADD,
  ACTION_VAR_SUB,
  ACTION_VAR_MUL,
  ACTION_VAR_DIV,
  ACTION_VAR_MOD,
  ACTION_VAR_RANDOM,
  ACTION_VAR_OBJECT_X,
  ACTION_VAR_OBJECT_Y,
  ACTION_VAR_OBJECT_FIX,
  ACTION_VAR_CELL_MAPPED,
  ACTION_VAR_SET,
  ACTION_VAR_PAL,
  ACTION_VAR_MOUSE_X,
  ACTION_VAR_MOUSE_Y,
  ACTION_VAR_CATCH,
  ACTION_VAR_COLLIDE,
  ACTION_VAR_INSIDE,
  ACTION_VAR_CELL_TRANSPARENT,

/* FKiSS 3.0 control structures */
  CONTROL = 1024,
  CONTROL_GOTO,
  CONTROL_GOSUB,
  CONTROL_GOTO_RANDOM,
  CONTROL_GOSUB_RANDOM,
  CONTROL_EQUAL,
  CONTROL_NOT_EQUAL,
  CONTROL_GREATER_THAN,
  CONTROL_LESS_THAN,
  CONTROL_ELSE,
  CONTROL_ENDIF,
/* Last action in the system */
  ACTION_NONEXISTENT
} KissActionType;

typedef void * KissValue;

#define KissCell(x) ((KissCell *) x)
#define KissObject(x) ((KissObject *) x)
#define KissString(x) ((gchar *) x)
#define KissValue(x) (* (int *) x)

typedef struct {
  KissActionType type;
  KissValue args[3];
} KissAction;

typedef struct {
  gchar *name;
  int value;
} KissVariable;

typedef struct {
  KissCell *cell1, *cell2;
  KissActionList collide, apart;
  int together;
} KissCollision;

typedef struct micro_timer {
  long when; /* relative to previous event */
  KissActionList what;
  struct micro_timer *next;
  struct micro_timer *prev;

} MicroTimer;

extern guint fk_timer_source; /* Source of timers */
extern MicroTimer *timers; /* Timers ready to fire */

extern GSList *variables; /* all variables */

extern GSList *cells; /* Global list of ALL KissCells */

extern KissObject *object; /* allocated array of KissObjects */

extern KissActionList *script; /* current action list */

extern guchar *palettes[PALETTES]; /* KCF palettes */

/* KissActions */

#define ERRORS &errors

extern KissActionList errors;      /* fKiSS errors... */
extern KissActionList initialize;  /* fKiSS initialize() */

extern KissActionList kalarms[ALARMS]; /* fKiSS alarm()s */
extern KissActionList ksets[SETS];     /* fKiSS set()s */

/* functions */

void set_timer(KissActionList alarm, long when);
void unset_timer(KissActionList alarm);
void reset_timers(void);

void render_all(void);
void cell_render(gpointer data, gpointer user_data);
KissCell *cell_new(KissObject *object, guchar *pal, gchar *filename);
void cell_delete(KissCell *cell);
KissCell *cell_intersect(KissCell *cell, int x, int y);
KissCell *cell_find(gchar *name);
void object_move(KissObject *object, int x, int y);
void object_set_location(KissObject *object, int x, int y);

int *var_find(gchar *name);
KissCollision *collision_find(KissCell *cell1, KissCell *cell2);
int cell_collision(KissCell *cell1, KissCell *cell2);

char *lha_open(const char *filename);
void lha_close();

void parse_file(const char *filename);

void parse_action(guchar *text);
void events(KissActionList list);
void trigger(KissAction *action);

void clean_up(void);
void change_cursor(GdkWindow *window, GdkCursorType type);
int case_fixup(char *filename);
FILE *open_any(const char *path, const char *mode);
void open_either(const char *filename);
KissCell *intersect(int x, int y);
void switch_view(int set);

