/*  GnomeKiss - A KiSS viewer for the GNOME desktop
    Copyright (C) 2000  Nick Lamb <njl195@zepler.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "kiss.h"

#define PREFIX "-xqifw="
#define TEMPLATE "/tmp/kiss-XXXXXX"

static int package_open= 0;
static char temp[]= TEMPLATE;

char *lha_open(const char *filename) {
  char options[]= PREFIX TEMPLATE;
  int child;

  lha_close();
  strcpy(temp, TEMPLATE); /* Must be replaced each time */
  mktemp(temp);
  if (mkdir(temp, 0700) != 0) {
    fprintf(stderr, strerror(errno));
    return NULL;
  }
  child = fork();
  if (child) {
    wait(NULL);
  } else {
    snprintf(options, sizeof(options), PREFIX "%s", temp);
    execlp("lha", "lha", options, filename, NULL);
  }
  package_open= 1;
  return g_strdup(temp);
}

void lha_close() {
  DIR *dir;
  struct dirent *dirent;
  
  if (package_open == 0) return;
  dir= opendir(temp);
  if (dir == NULL) return;
  chdir(temp);

  while ((dirent= readdir(dir))) {
    if (dirent->d_name[0] != '.') {
      if ( unlink(dirent->d_name) );
    }
  }

  closedir(dir);
  rmdir(temp);
  package_open= 0;
}


