/*  GnomeKiss - A KiSS viewer for the GNOME desktop
    Copyright (C) 2000  Nick Lamb <njl195@zepler.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <limits.h>
#include <sys/time.h>
#include <unistd.h>

#include "kiss.h"

guint fk_timer_source= 0;
MicroTimer *timers = NULL; /* Timers ready to fire */

/* internals */

static gboolean check_timers(gpointer data);
static MicroTimer timer_zero;
static struct timeval lasttime;

#if 0
static void print_timer(char *message) {
  MicroTimer *timer;

  fprintf(stderr, "%s: timer_zero.next->what=%p\n", message, timer_zero.next->what);
  fprintf(stderr, "%s: timer_zero.prev->what=%p\n", message, timer_zero.prev->what);
  for (timer= timers; timer != NULL; timer=timer->next) {
    fprintf(stderr, "%s: {%p,%p,%p} %ldms [%p]..\n", message,
                                  timer->prev, timer, timer->next,
                                  timer->when, timer->what);

    if (timer->what == NULL) {
      break;
    }
  }
}
#endif

static gboolean check_timers(gpointer data) {
  GSList *actions;
  struct timeval thistime;
  long count;

  gettimeofday(&thistime, NULL);
  count= ( thistime.tv_sec - lasttime.tv_sec ) * 1000;
  count+= ( thistime.tv_usec - lasttime.tv_usec ) / 1000;

/* FIXME this is the perfect place to offer 2x 5x 10x slowdown */

  if (count == 0) return TRUE;

  while (timers->what != NULL && timers->when < count) {
    count-= timers->when;
    actions= timers->what;
    timer_zero.next= timers->next;
    timers->next->prev= &timer_zero;
    timers= timer_zero.next;
    events(actions);
  }

  lasttime.tv_sec= thistime.tv_sec;
  lasttime.tv_usec= thistime.tv_usec;
  if (timers->what) {
    timers->when-= count;
    return TRUE;
  } else {
    fk_timer_source= 0; /* source gone */
    return FALSE;
  }
}

void set_timer(GSList *what, long when) {
  long count= 0;
  MicroTimer *timer, *new_timer, *old_timer= NULL;

  if (what == NULL) {
    return;
  } else if (when == 0 || when >= 32767) {
    unset_timer(what);
    return;
  }

  if (fk_timer_source == 0) {
    fk_timer_source= gtk_timeout_add(25, (GSourceFunc) check_timers, NULL);
    gettimeofday(&lasttime, NULL);
  }

  new_timer= g_new(MicroTimer, 1);
  new_timer->what= what;

  for (timer= timers; timer != NULL; timer=timer->next) {
    count+= timer->when;

    if (new_timer && count > when) {
      new_timer->when= when + timer->when - count;
      new_timer->prev= timer->prev;
      new_timer->next= timer;
      timer->when= count - when;
      timer->prev->next= new_timer;
      timer->prev= new_timer;
      new_timer= NULL;
    }

    if (timer->what == what) {
      old_timer= timer;
      timer->prev->next= timer->next;
      timer->next->when+= timer->when;
      timer->next->prev= timer->prev;
      timer= timer->prev;
    }

    if (timer->what == NULL) { /* timer_zero */
      timer->when= LONG_MAX / 2;
      break;
    }
  }

  if (old_timer) g_free(old_timer); /* clean up */
  timers= timer_zero.next;
}

void unset_timer(GSList *alarm) {
  MicroTimer *timer, *old_timer = NULL;

  if (alarm == NULL) {
    return;
  }
  for (timer= timers; timer != NULL; timer=timer->next) {
    if (timer->what == alarm) {
      old_timer= timer;
      timer->prev->next= timer->next;
      timer->next->when+= timer->when;
      timer->next->prev= timer->prev;
      timer= timer->prev;
    }
    if (timer->what == NULL) { /* timer_zero */
      timer->when= LONG_MAX / 2;
      break;
    }
  }
  if (old_timer) {
    g_free(old_timer); /* clean up */
  }
  timers= timer_zero.next;
}

void reset_timers(void) {
  MicroTimer *next;
  
  while (timers && timers->what) {
    next= timers->next;
    g_free(timers);
    timers= next;
  }

  timer_zero.when= LONG_MAX / 2;
  timer_zero.what= NULL;
  timer_zero.next= &timer_zero;
  timer_zero.prev= &timer_zero;
  timers= timer_zero.next;

  if (fk_timer_source > 0) {
    g_source_remove(fk_timer_source);
    fk_timer_source= 0;
  }
}
