/*  GnomeKiss - A KiSS viewer for the GNOME desktop
    Copyright (C) 2000  Nick Lamb <njl195@zepler.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <dirent.h>
#include "kiss.h"

int global_lock= KISS_DONE;

static void free_action_list(GSList *list);

static void free_action_list(GSList *list) {
  while (list != NULL) {
    if (list->data)
       g_free((KissAction *) list->data);
    list= g_slist_next(list);
  }
}

void render_all(void) {
  global_lock= KISS_PENDING;
  if (area != NULL) {
    gtk_widget_queue_draw(area);
  }
}

/* Only works for filename in the current directory, deliberately */
int case_fixup(char *filename) {
  DIR *dir;
  struct dirent *dirent;
  size_t len;
  
  dir= opendir(".");
  if (dir == NULL) return 0;
  len= strlen(filename);

  while ((dirent= readdir(dir))) {
    if (!g_strcasecmp(filename, dirent->d_name)) {
      strcpy(filename, dirent->d_name);
      closedir(dir);
      return 1;
    }
  }

  closedir(dir);
  return 0;
}

FILE *open_any(const char *path, const char *mode) {
  FILE *fp = NULL;
  char *temp;

  temp = g_strdup(path);

  fp= fopen(temp, mode);
  if (fp == NULL && case_fixup(temp)) {
    fp= fopen(temp, mode);
  }
  
  g_free(temp);
  if (fp == NULL)
    g_message("File not found \"%s\"", path);
  return fp;
}

void change_cursor(GdkWindow *window, GdkCursorType type) {
  GdkCursor *cursor;

  cursor = gdk_cursor_new(type);
  gdk_window_set_cursor(window, cursor);
  gdk_cursor_destroy(cursor);
}

/* FIXME I think we're leaking a lot of linked lists */

void clean_up() {
  GSList *list;
  int k;
  
  /* timers */
  reset_timers();

  /* Remove RGB buffer */
  if (rgb_buf != NULL) {
    g_free(rgb_buf);
    rgb_buf= NULL;
  }

  /* free Alarms */
  for (k=0; k < ALARMS; ++k) {
    free_action_list(kalarms[k]);
    kalarms[k]= NULL;
  }

  /* free Sets */
  for (k=0; k < SETS; ++k) {
    free_action_list(ksets[k]);
    ksets[k]= NULL;
  }

  /* free Initialize */
  free_action_list(initialize);
  initialize= NULL;

  /* free Errors */
  free_action_list(errors);
  errors= NULL;

  /* free Cells */
  for (list= cells; list != NULL; list= g_slist_next(list)) {
    cell_delete((KissCell *) list->data);
  }
  cells= NULL; /* empty now */

  /* free Objects */
  if (object!= NULL) {
    g_free(object);
  }
  object= NULL;
}

KissCell *intersect(int x, int y) {
  GSList *list;
  KissCell *final, *current;

  final= NULL;
  for (list= cells; list != NULL; list= g_slist_next(list)) {
    current= cell_intersect((KissCell *) list->data, x, y);
    if (current) final= current;
  }
  return final;
}

void open_either(const char *filename) {
  int len;
  char *temp;
  
  len= strlen(filename);
  if (len > 4 && g_strcasecmp(filename + len - 4, ".lzh") == 0) {
    temp= lha_open(filename);
    gtk_file_selection_complete(GTK_FILE_SELECTION(config_open), temp);
    gtk_file_selection_complete(GTK_FILE_SELECTION(config_open), "*.cnf");
    gtk_widget_show (config_open);
    g_free(temp);
  } else {
    parse_file(filename);
  }
  gtk_window_set_title(GTK_WINDOW(app), filename);

}

void switch_view(int set) {
  if (set < 0 || set >= SETS) return;
  if (set == view) return;
  view= set;
  events(ksets[set]);
}
